package com.dougmccune.controls
{
	import com.dougmccune.effects.YoyoMove;
	
	import flash.display.BlendMode;
	import flash.events.Event;
	
	import flexlib.controls.Fire;
	
	import mx.controls.Label;
	import mx.core.IDataRenderer;
	import mx.core.UIComponent;
	import mx.effects.easing.Cubic;
	import mx.events.EffectEvent;
	
	import qs.controls.CachedLabel;

	[Event("loaded")]
	
	public class BouncingBurningTile extends UIComponent implements IDataRenderer
	{
		[Bindable]
		public var tile:BurningBitmapTile;
		
		private var _burning:Boolean = false;
		
		private var _data:Object;
		private var bounce:YoyoMove;
		
	
		private var _showLabel:Boolean = false;
		
		[Bindable]
		public var label:CachedLabel;
		
		private var minYoyoCount:int = 0;
		
		public function get showLabel():Boolean {
			return _showLabel;
		}
		
		[Bindable]
		public function set showLabel(value:Boolean):void {
			this._showLabel = value;
			
			invalidateDisplayList();
		}
		
		override protected function measure():void {
			measuredWidth = tile.measuredWidth;
			measuredHeight = tile.measuredHeight;
		}
		
		override protected function createChildren():void {
			tile = new BurningBitmapTile();
			tile.addEventListener("loaded", tileLoaded);
			tile.styleName = this;
			
			tile.setStyle("borderAlpha", 0);

			tile.burning = _burning;
			tile.data = _data;
			
			label = new CachedLabel();
			label.useCache = "on";
			label.data = data.label;
			label.setStyle("color", 0xffffff);
			label.setStyle("fontSize", 18);
		
			addChild(tile);
			addChild(label);
		}
		
		private function tileLoaded(event:Event):void {
			invalidateSize();
			invalidateDisplayList();
			invalidateSize();
			
			dispatchEvent(new Event("loaded"));		
		}
		
		public function get data():Object
		{
			return _data;
		}
		
		public function set data(value:Object):void
		{
			_data = value;
			if(tile) {
				tile.data = value;
			}
			
			if(label) {
				label.data = value.label;
			}
		}
		
		override protected function updateDisplayList(unscaledWidth:Number, unscaledHeight:Number):void
		{
			super.updateDisplayList(unscaledWidth, unscaledHeight);
			
			label.visible = _showLabel;
			label.move(unscaledWidth/2 - label.getExplicitOrMeasuredWidth()/2, -label.getExplicitOrMeasuredHeight());
			label.setActualSize(label.getExplicitOrMeasuredWidth(), label.getExplicitOrMeasuredHeight());
			tile.setActualSize(unscaledWidth, unscaledHeight);
		}
		
		
		public function bounceImage(count:int, xTo:Number, yTo:Number, minCount:int=0):void {
			tile.burning = false;
		
			minYoyoCount = minCount;
			
			if(!bounce) {
				bounce = new YoyoMove(tile);
				bounce.addEventListener(EffectEvent.EFFECT_END, reinstateBurning);
				
				bounce.easingFunction = Cubic.easeOut;
				bounce.duration = 300;
			}
			
			bounce.yFrom = 0;
			bounce.xFrom = 0;
			bounce.xTo = xTo;
			bounce.yTo = yTo;
			bounce.yoyoCount = count;

			bounce.play();
		}
		
		private function reinstateBurning(event:Event):void {
			tile.burning = _burning;
		}
		
		public function stopBouncing():void {
			if(bounce) {
				
				if(bounce.currentYoyoCount < minYoyoCount) {
					bounce.yoyoCount = minYoyoCount - bounce.currentYoyoCount + 1;
				}
				else {
					bounce.end();
				}
			}
		}
		
		[Bindable]
		public function set burning(value:Boolean):void {
			_burning = value;
			
			if(tile && bounce && !bounce.isPlaying) {
				tile.burning = value;
			}
		}
		
		public function get burning():Boolean {
			return _burning;
		}
	}
}